<?php

namespace Drupal\social_auth_bitbucket\Controller;

use Drupal\Core\Messenger\MessengerInterface;
use Drupal\social_api\Plugin\NetworkManager;
use Drupal\social_auth\Controller\OAuth2ControllerBase;
use Drupal\social_auth\SocialAuthDataHandler;
use Drupal\social_auth\User\UserAuthenticator;
use Drupal\social_auth_bitbucket\BitbucketAuthManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Returns responses for Social Auth Bitbucket routes.
 */
class BitbucketAuthController extends OAuth2ControllerBase {

  /**
   * BitbucketAuthController constructor.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\social_api\Plugin\NetworkManager $network_manager
   *   Used to get an instance of social_auth_bitbucket network plugin.
   * @param \Drupal\social_auth\User\UserAuthenticator $user_authenticator
   *   Manages user login/registration.
   * @param \Drupal\social_auth_bitbucket\BitbucketAuthManager $bitbucket_manager
   *   Used to manage authentication methods.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request
   *   Used to access GET parameters.
   * @param \Drupal\social_auth\SocialAuthDataHandler $data_handler
   *   The Social Auth data handler.
   */
  public function __construct(MessengerInterface $messenger,
                              NetworkManager $network_manager,
                              UserAuthenticator $user_authenticator,
                              BitbucketAuthManager $bitbucket_manager,
                              RequestStack $request,
                              SocialAuthDataHandler $data_handler) {

    parent::__construct('Social Auth Bitbucket', 'social_auth_bitbucket', $messenger, $network_manager, $user_authenticator, $bitbucket_manager, $request, $data_handler);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger'),
      $container->get('plugin.network.manager'),
      $container->get('social_auth.user_authenticator'),
      $container->get('social_auth_bitbucket.manager'),
      $container->get('request_stack'),
      $container->get('social_auth.data_handler')
    );
  }

  /**
   * Response for path 'user/login/bitbucket/callback'.
   *
   * Bitbucket returns the user here after user has authenticated in Bitbucket.
   */
  public function callback() {
    // Checks if authentication failed.
    if ($this->request->getCurrentRequest()->query->has('error')) {
      $this->messenger->addError($this->t('You could not be authenticated.'));

      return $this->redirect('user.login');
    }

    /* @var \Stevenmaguire\OAuth2\Client\Provider\BitbucketResourceOwner|null $profile */
    $profile = $this->processCallback();

    // If authentication was successful.
    if ($profile !== NULL) {

      // Gets (or not) extra initial data.
      $data = $this->userAuthenticator->checkProviderIsAssociated($profile->getId()) ? NULL : $this->providerManager->getExtraDetails();

      // Gets email address.
      $emails = $this->providerManager->requestEndPoint('GET', '/2.0/user/emails');
      $email = '';
      foreach ($emails['values'] as $email_info) {
        if ($email_info['is_primary']) {
          $email = $email_info['email'];
          break;
        }
      }

      // If user information could be retrieved.
      return $this->userAuthenticator->authenticateUser($profile->getName(), $email, $profile->getId(), $this->providerManager->getAccessToken(), $profile->toArray()['links']['avatar']['href'], $data);
    }

    return $this->redirect('user.login');
  }

}
